/****************************************************************************
**
** Copyright (C) 2019 Luxoft Sweden AB
** Copyright (C) 2018 Pelagicore AG
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtIvi module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL-QTAS$
** Commercial License Usage
** Licensees holding valid commercial Qt Automotive Suite licenses may use
** this file in accordance with the commercial license agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and The Qt Company.  For
** licensing terms and conditions see https://www.qt.io/terms-conditions.
** For further information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
** SPDX-License-Identifier: LGPL-3.0
**
****************************************************************************/

#include "qiviserviceobject.h"

#include <QUuid>

QT_BEGIN_NAMESPACE

/*!
    \class QIviServiceObject
    \inmodule QtIviCore
    \brief Provides the connection point to a Backend Service.

    QIviServiceObject provides you with a list of interfaces that the Backend implements.

    Use interfaceInstance() to obtain a QObject that implements this interface. The
    interface returned may contain signals that a Feature implementing this interface
    needs to be connected to.

    \sa QIviAbstractFeature
*/


/*!
    Constructor.

    \a parent is passed on to \l QObject.
*/
QIviServiceObject::QIviServiceObject(QObject *parent)
    : QObject(parent)
    , m_id(QUuid::createUuid().toString())
{
}

/*!
    \property QIviServiceObject::id
    \brief A unique ID for the service object instance.

    Each service object has a unique ID. When subclassing, the id()
    function can be overloaded to modify how this ID is generated.
*/

/*!
    The id() function can be overloaded to modify how the unique ID is generated,
    for use by this service object.

    By default, QUuid::createUuid() is used.
*/
QString QIviServiceObject::id() const
{
    return m_id;
}

QT_END_NAMESPACE
